@@ -9,7 +9,7 @@ import qualified Data.ByteString.Char8 as BS
 import Data.Streaming.Network (bindPortTCP)
 import Network (Socket, sClose)
 import qualified Network.HTTP.Client as H
-import Network.Wai.Application.Classic hiding ((</>), (+++))
+import Network.Wai.Application.Classic hiding ((</>))
 import Network.Wai.Handler.Warp
 import Network.Wai.Logger
 import System.Exit (ExitCode(..), exitSuccess)
@@ -21,6 +21,8 @@ import System.Posix.Signals (sigCHLD)
 import Program.Mighty
 import WaiApp
 
+import qualified Network.Wai.Middleware.Push.Referer as P
+
 #ifdef HTTP_OVER_TLS
 import Control.Concurrent.Async (concurrently)
 import Control.Monad (void)
@@ -65,11 +67,12 @@ server opt rpt route = reportDo rpt $ do
     ap <- initLogger FromSocket logtype zdater
     let lgr = apacheLogger ap
         remover = logRemover ap
+        pushlgr = serverpushLogger ap
     mgr <- getManager opt
     setHandlers opt rpt svc remover rdr
 
     report rpt "Mighty started"
-    runInUnboundThread $ mighty opt rpt svc lgr mgr rdr tlsSetting
+    runInUnboundThread $ mighty opt rpt svc lgr pushlgr mgr rdr tlsSetting
     report rpt "Mighty retired"
     finReporter rpt
     remover
@@ -141,10 +144,11 @@ ifRouteFileIsValid rpt opt act = case opt_routing_file opt of
 ----------------------------------------------------------------
 
 mighty :: Option -> Reporter -> Service
-       -> ApacheLogger -> ConnPool -> RouteDBRef
+       -> ApacheLogger -> ServerPushLogger
+       -> ConnPool -> RouteDBRef
        -> TLSSettings
        -> IO ()
-mighty opt rpt svc lgr mgr rdr _tlsSetting
+mighty opt rpt svc lgr pushlgr mgr rdr _tlsSetting
   = reportDo rpt $ case svc of
     HttpOnly s  -> runSettingsSocket setting s app
 #ifdef HTTP_OVER_TLS
@@ -156,7 +160,8 @@ mighty opt rpt svc lgr mgr rdr _tlsSetting
     _ -> error "never reach"
 #endif
   where
-    app req = fileCgiApp cspec filespec cgispec revproxyspec rdr req
+    app = P.pushOnReferer P.defaultSettings
+                          (fileCgiApp cspec filespec cgispec revproxyspec rdr)
     debug = opt_debug_mode opt
     -- We don't use setInstallShutdownHandler because we may use
     -- two sockets for HTTP and HTTPS.
@@ -168,6 +173,7 @@ mighty opt rpt svc lgr mgr rdr _tlsSetting
             $ setFileInfoCacheDuration 10
             $ setServerName      serverName
             $ setLogger          lgr
+            $ setServerPushLogger pushlgr
             defaultSettings
     serverName = BS.pack $ opt_server_name opt
     cspec = ClassicAppSpec {
@@ -236,4 +242,4 @@ getManager opt = H.newManager H.defaultManagerSettings {
   where
     responseTimeout
       | opt_proxy_timeout opt == 0 = H.managerResponseTimeout H.defaultManagerSettings
-      | otherwise                  = Just (opt_proxy_timeout opt * 1000000) -- micro seconds
+      | otherwise                  = H.responseTimeoutMicro (opt_proxy_timeout opt * 1000000) -- micro seconds
